#!/usr/bin/env python
"""This file is automatically generated by generate_version_info
It uses the current working tree to determine the revision.
So don't edit it. :)
"""

version_info = {'branch_nick': u'vmbuilder',
 'build_date': '2011-12-22 10:49:13 -0600',
 'clean': None,
 'date': '2011-12-22 10:11:20 -0600',
 'revision_id': 'serge.hallyn@canonical.com-20111222161120-73vtdj1lh029ax92',
 'revno': '475'}

revisions = {}

file_revisions = {}


if __name__ == '__main__':
    print 'revision: %(revno)s' % version_info
    print 'nick: %(branch_nick)s' % version_info
    print 'revision id: %(revision_id)s' % version_info
